package org.commonjava.tensor.data;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.maven.graph.common.RelationshipType;
import org.apache.maven.graph.common.ref.ProjectRef;
import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.EProjectGraph;
import org.apache.maven.graph.effective.EProjectRelationships;
import org.apache.maven.graph.effective.ref.EProjectKey;
import org.apache.maven.graph.effective.rel.ProjectRelationship;

public interface TensorDataManager
{

    public static final String PROJECT_RELATIONSHIPS_CACHE = "tensor-effective-project-relationships";

    public static final String GRAPH_CACHE = "tensor-project-graphs";

    boolean storeGraph( boolean skipIfExists, EProjectGraph graph )
        throws TensorDataException;

    boolean storeRelationships( boolean skipIfExists, EProjectRelationships relationships )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( ProjectRelationship<?>... relationships )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( Collection<ProjectRelationship<?>> relationships )
        throws TensorDataException;

    EProjectGraph getProjectGraph( EProjectKey key )
        throws TensorDataException;

    EProjectGraph getProjectGraph( ProjectVersionRef ref )
        throws TensorDataException;

    List<ProjectVersionRef> getAncestry( ProjectVersionRef source )
        throws TensorDataException;

    ProjectVersionRef getParent( ProjectVersionRef source )
        throws TensorDataException;

    Set<ProjectVersionRef> getKnownChildren( ProjectVersionRef parent )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithExactSource( ProjectVersionRef source,
                                                                          RelationshipType... types )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithExactTarget( ProjectVersionRef target,
                                                                          RelationshipType... types )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithGATarget( ProjectRef target, RelationshipType... types )
        throws TensorDataException;

    EProjectRelationships getProjectRelationships( EProjectKey key )
        throws TensorDataException;

    EProjectRelationships getProjectRelationships( ProjectVersionRef projectRef )
        throws TensorDataException;

    void reindex()
        throws TensorDataException;

    boolean contains( ProjectVersionRef ref );

    Set<ProjectVersionRef> getAllStoredProjectRefs()
        throws TensorDataException;
}