package org.commonjava.tensor.data.idx;

import java.util.HashSet;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.maven.graph.common.ref.ProjectRef;
import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.rel.ProjectRelationship;
import org.commonjava.tensor.data.TensorDataException;
import org.commonjava.tensor.data.store.IndexStore;
import org.commonjava.tensor.data.store.IndexStoreFactory;

import com.google.gson.reflect.TypeToken;

@javax.enterprise.context.ApplicationScoped
@Named( AllByTargetIndex.ALL_BY_TARGET_NAME )
public class AllByTargetIndex
    implements Index<ProjectRef, Set<ProjectRelationship<?>>>
{

    public static final String ALL_BY_TARGET_NAME = "tensorIdx-all-by-target";

    //    private final Logger logger = new Logger( getClass() );

    @Inject
    private IndexStoreFactory factory;

    private IndexStore<ProjectRef, Set<ProjectRelationship<?>>> store;

    @PostConstruct
    public void injectCache()
    {
        store = factory.getStore( ALL_BY_TARGET_NAME, ProjectRef.class, new TypeToken<Set<ProjectRelationship<?>>>()
        {
        } );
    }

    public AllByTargetIndex()
    {
    }

    public AllByTargetIndex( final IndexStore<ProjectRef, Set<ProjectRelationship<?>>> store )
    {
        this.store = store;
    }

    @Override
    public boolean update( final Object value )
        throws TensorDataException
    {
        if ( value instanceof ProjectRelationship<?> )
        {
            final ProjectRelationship<?> rel = (ProjectRelationship<?>) value;
            final ProjectRef target = rel.getTarget();

            final Set<ProjectRef> targets = new HashSet<ProjectRef>();
            targets.add( target );

            if ( target instanceof ProjectVersionRef )
            {
                targets.add( new ProjectRef( target.getGroupId(), target.getArtifactId() ) );
            }

            boolean updated = false;
            for ( final ProjectRef t : targets )
            {
                Set<ProjectRelationship<?>> set = store.get( t );
                if ( set == null )
                {
                    set = new HashSet<ProjectRelationship<?>>();
                }

                final boolean u = set.add( rel );
                if ( u )
                {
                    store.store( t, set );
                }

                updated = u || updated;
            }

            return updated;
        }

        return false;
    }

    @Override
    public Set<ProjectRelationship<?>> get( final ProjectRef query )
        throws TensorDataException
    {
        return store.get( query );
    }

    @Override
    public void clear()
        throws TensorDataException
    {
        store.clear();
    }

}
