package org.commonjava.tensor.data.idx;

import java.util.HashSet;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.rel.ParentRelationship;
import org.commonjava.tensor.data.TensorDataException;
import org.commonjava.tensor.data.store.IndexStore;
import org.commonjava.tensor.data.store.IndexStoreFactory;

import com.google.gson.reflect.TypeToken;

@javax.enterprise.context.ApplicationScoped
@Named( ChildrenOfIndex.CHILDREN_OF_INDEX_NAME )
public class ChildrenOfIndex
    implements Index<ProjectVersionRef, Set<ProjectVersionRef>>
{

    public static final String CHILDREN_OF_INDEX_NAME = "tensorIdx-children-of";

    @Inject
    private IndexStoreFactory factory;

    private IndexStore<ProjectVersionRef, Set<ProjectVersionRef>> store;

    @PostConstruct
    public void injectCache()
    {
        store =
            factory.getStore( CHILDREN_OF_INDEX_NAME, ProjectVersionRef.class, new TypeToken<Set<ProjectVersionRef>>()
            {
            } );
    }

    public ChildrenOfIndex()
    {
    }

    public ChildrenOfIndex( final IndexStore<ProjectVersionRef, Set<ProjectVersionRef>> store )
    {
        this.store = store;
    }

    @Override
    public boolean update( final Object value )
        throws TensorDataException
    {
        if ( value instanceof ParentRelationship )
        {
            final ParentRelationship rel = (ParentRelationship) value;
            Set<ProjectVersionRef> children = store.get( rel.getTarget() );
            if ( children == null )
            {
                children = new HashSet<ProjectVersionRef>();
            }

            final boolean updated = children.add( rel.getDeclaring() );
            if ( updated )
            {
                store.store( rel.getTarget(), children );
            }
        }

        return false;
    }

    @Override
    public Set<ProjectVersionRef> get( final ProjectVersionRef query )
        throws TensorDataException
    {
        return store.get( query );
    }

    @Override
    public void clear()
        throws TensorDataException
    {
        store.clear();
    }

}
