package org.commonjava.tensor.data.idx;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.rel.ParentRelationship;
import org.commonjava.tensor.data.TensorDataException;
import org.commonjava.tensor.data.store.IndexStore;
import org.commonjava.tensor.data.store.IndexStoreFactory;

@javax.enterprise.context.ApplicationScoped
@Named( ParentOfIndex.PARENT_OF_INDEX_NAME )
public class ParentOfIndex
    implements Index<ProjectVersionRef, ParentRelationship>
{

    public static final String PARENT_OF_INDEX_NAME = "tensorIdx-parent-of";

    @Inject
    private IndexStoreFactory container;

    private IndexStore<ProjectVersionRef, ParentRelationship> cache;

    @PostConstruct
    public void injectCache()
    {
        cache = container.getStore( PARENT_OF_INDEX_NAME, ProjectVersionRef.class, ParentRelationship.class );
    }

    public ParentOfIndex()
    {
    }

    public ParentOfIndex( final IndexStore<ProjectVersionRef, ParentRelationship> cache )
    {
        this.cache = cache;
    }

    @Override
    public boolean update( final Object value )
        throws TensorDataException
    {
        if ( value instanceof ParentRelationship )
        {
            final ParentRelationship rel = (ParentRelationship) value;
            final ParentRelationship old = cache.store( rel.getDeclaring(), rel );
            return old == null;
        }

        return false;
    }

    @Override
    public ParentRelationship get( final ProjectVersionRef query )
        throws TensorDataException
    {
        return cache.get( query );
    }

    @Override
    public void clear()
        throws TensorDataException
    {
        cache.clear();
    }

}
